@@ -1,12 +1,10 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
 # Copyright 2011 Alexander Zangerl <az@snafu.priv.at>
 # Copyright 2012 edso (ssh_config added)
 #
-# $Id: sshbackend.py,v 1.2 2011/12/31 04:44:12 az Exp $
-#
 # This file is part of duplicity.
 #
 # Duplicity is free software; you can redistribute it and/or modify it
@@ -39,6 +37,7 @@ import warnings
 from binascii import hexlify
 
 import duplicity.backend
+from duplicity import progress
 from duplicity import config
 from duplicity import util
 from duplicity.errors import BackendException
@@ -154,25 +153,53 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname,
             wanted = logging.CRITICAL
         ours.setLevel(wanted)
 
-        # load known_hosts files
+        # load user/local known_hosts files
         # paramiko is very picky wrt format and bails out on any problem...
+        global_known_hosts = u"/etc/ssh/ssh_known_hosts"
+        m = re.search(r"""
+                      ^(?:.+\s+)?
+                      (?:-oGlobalKnownHostsFile=)
+                      (
+                          ([\"'])
+                          ([^\\2]+)
+                          \\2
+                          |
+                          [\S]+
+                      )
+                      """,
+                      config.ssh_options, re.VERBOSE)
+        if (m is not None):
+            global_known_hosts = m.group(3) if m.group(3) else m.group(1)
         try:
-            if os.path.isfile(u"/etc/ssh/ssh_known_hosts"):
-                self.client.load_system_host_keys(u"/etc/ssh/ssh_known_hosts")
+            if os.path.isfile(global_known_hosts):
+                self.client.load_system_host_keys(global_known_hosts)
         except Exception as e:
-            raise BackendException(u"could not load /etc/ssh/ssh_known_hosts, "
-                                   u"maybe corrupt?")
+            raise BackendException(f"could not load {global_known_hosts}, maybe corrupt?")
+
+        user_known_hosts = os.path.expanduser(u"~/.ssh/known_hosts")
+        m = re.search(r"""
+                      ^(?:.+\s+)?
+                      (?:-oUserKnownHostsFile=)
+                      (
+                          ([\"'])
+                          ([^\\2]+)
+                          \\2
+                          |
+                          [\S]+
+                      )
+                      """,
+                      config.ssh_options, re.VERBOSE)
+        if (m is not None):
+            user_known_hosts = m.group(3) if m.group(3) else m.group(1)
         try:
             # use load_host_keys() to signal it's writable to paramiko
             # load if file exists or add filename to create it if needed
-            file = os.path.expanduser(u'~/.ssh/known_hosts')
-            if os.path.isfile(file):
-                self.client.load_host_keys(file)
+            if os.path.isfile(user_known_hosts):
+                self.client.load_host_keys(user_known_hosts)
             else:
-                self.client._host_keys_filename = file
+                self.client._host_keys_filename = user_known_hosts
         except Exception as e:
-            raise BackendException(u"could not load ~/.ssh/known_hosts, "
-                                   u"maybe corrupt?")
+            raise BackendException(f"could not load {user_known_hosts}, maybe corrupt?")
 
         u""" the next block reorganizes all host parameters into a
         dictionary like SSHConfig does. this dictionary 'self.config'
@@ -203,11 +230,20 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname,
         else:
             self.config.update({u'port': 22})
         # parse ssh options for alternative ssh private key, identity file
-        m = re.search(r"^(?:.+\s+)?(?:-oIdentityFile=|-i\s+)(([\"'])([^\\2]+)\\2|[\S]+).*",
-                      config.ssh_options)
+        m = re.search(r"""
+                      ^(?:.+\s+)?
+                      (?:-oIdentityFile=|-i\s+)
+                      (([\"'])
+                      (
+                          [^\\2]+)\\2
+                          |
+                          [\S]+
+                      )
+                      """,
+                      config.ssh_options, re.VERBOSE)
         if (m is not None):
             keyfilename = m.group(3) if m.group(3) else m.group(1)
-            self.config[u'identityfile'] = keyfilename
+            self.config[u'identityfile'] = keyfilename.strip(u'\'\"')
         # ensure ~ is expanded and identity exists in dictionary
         if u'identityfile' in self.config:
             if not isinstance(self.config[u'identityfile'], list):
@@ -269,9 +305,8 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname,
             # move to the appropriate directory, possibly after creating it and its parents
             dirs = self.remote_dir.split(os.sep)
             if len(dirs) > 0:
-                if not dirs[0]:
-                    dirs = dirs[1:]
-                    dirs[0] = u'/' + dirs[0]
+                if dirs[0] == u'':
+                    dirs[0] = u'/'
                 for d in dirs:
                     if (d == u''):
                         continue
@@ -295,7 +330,7 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname,
 
     def _put(self, source_path, remote_filename):
         # remote_filename is a byte object, not str or unicode
-        remote_filename = remote_filename.decode(u"utf-8")
+        remote_filename = util.fsdecode(remote_filename)
         if self.use_scp:
             f = open(source_path.name, u'rb')
             try:
@@ -317,18 +352,24 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname,
             response = chan.recv(1)
             if (response != b"\0"):
                 raise BackendException(b"scp remote error: %b" % chan.recv(-1))
-            chan.sendall(f.read() + b'\0')
+            file_pos = 0
+            file_size = fstat.st_size
+            while file_pos < file_size:
+                chan.sendall(f.read(16384))
+                file_pos = f.tell()
+                progress.report_transfer(file_pos, file_size)
+            chan.sendall(b'\0')
             f.close()
             response = chan.recv(1)
             if (response != b"\0"):
                 raise BackendException(u"scp remote error: %s" % chan.recv(-1))
             chan.close()
         else:
-            self.sftp.put(source_path.name, remote_filename)
+            self.sftp.put(source_path.name, remote_filename, callback=progress.report_transfer)
 
     def _get(self, remote_filename, local_path):
         # remote_filename is a byte object, not str or unicode
-        remote_filename = remote_filename.decode(u"utf-8")
+        remote_filename = util.fsdecode(remote_filename)
         if self.use_scp:
             try:
                 chan = self.client.get_transport().open_session()
@@ -385,6 +426,8 @@ Are you sure you want to continue connecting (yes/no)? """ % (hostname,
             return self.sftp.listdir()
 
     def _delete(self, filename):
+        # filename is a byte object, not str or unicode
+        filename = util.fsdecode(filename)
         # In scp mode unavoidable quoting issues will cause failures if
         # filenames containing single quotes are encountered.
         if self.use_scp:
