@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf8 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # duplicity -- Encrypted bandwidth efficient backup
 #
@@ -21,7 +21,11 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# See http://www.nongnu.org/duplicity for more information.
+# For more information see
+#  http://duplicity.us
+#  or
+#  http://duplicity.gitlab.io
+# .
 # Please send mail to me or the mailing list if you find bugs or have
 # any suggestions.
 
@@ -252,7 +256,8 @@ def restart_position_iterator(tarblock_iter):
                 if not last_block and not tarblock_iter.previous_block:
                     break
                 # Only check block number if last_block is also a number
-                if last_block and tarblock_iter.previous_block > last_block:
+                if (last_block and tarblock_iter.previous_block
+                        and tarblock_iter.previous_block > last_block):
                     break
             if tarblock_iter.previous_index > last_index:
                 log.Warn(_(u"File %s complete in backup set.\n"
@@ -383,7 +388,7 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
         mf = config.restart.last_backup.get_local_manifest()
         config.restart.checkManifest(mf)
         config.restart.setLastSaved(mf)
-        if not (config.s3_use_deep_archive or config.s3_use_glacier):
+        if not (config.s3_use_deep_archive or config.s3_use_glacier or config.s3_use_glacier_ir):
             validate_encryption_settings(config.restart.last_backup, mf)
         else:
             log.Warn(_(u"Skipping encryption validation due to glacier/deep storage"))
@@ -423,13 +428,21 @@ def write_multivol(backup_type, tarblock_iter, man_outfp, sig_outfp, backend):
         tarblock_iter.remember_next_index()  # keep track of start index
 
         # Create volume
+        try:
+            log.Debug(u"BACKEND: " + str(config.backend))
+        except:
+            pass
+        log.Debug(u"***CREATING VOLUME***")
+
         vol_num += 1
         dest_filename = file_naming.get(backup_type, vol_num,
                                         encrypted=config.encryption,
                                         gzipped=config.compression)
         tdp = dup_temp.new_tempduppath(file_naming.parse(dest_filename))
+        log.Debug(u"FILENAME: " + str(tdp.name))
 
         # write volume
+        log.Debug(u"***WRITING VOLUME***")
         if config.encryption:
             at_end = gpg.GPGWriteFile(tarblock_iter, tdp.name, config.gpg_profile,
                                       config.volsize)
@@ -809,28 +822,33 @@ def restore_get_enc_fileobj(backend, filename, volume_info):
     with --ignore-errors set continue on hash mismatch
 
     """
-    parseresults = file_naming.parse(filename)
-    tdp = dup_temp.new_tempduppath(parseresults)
-    backend.get(filename, tdp)
+    for n in range(1, config.num_retries + 1):
+        u""" get the remote file """
+        parseresults = file_naming.parse(filename)
+        tdp = dup_temp.new_tempduppath(parseresults)
+        backend.get(filename, tdp)
 
-    u""" verify hash of the remote file """
-    verified, hash_pair, calculated_hash = restore_check_hash(volume_info, tdp)
-    if not verified:
-        error_msg = u"%s\n %s\n %s\n %s\n" % (
-            _(u"Invalid data - %s hash mismatch for file:") %
-            hash_pair[0],
-            util.fsdecode(filename),
-            _(u"Calculated hash: %s") % calculated_hash,
-            _(u"Manifest hash: %s") % hash_pair[1]
-        )
+        u""" verify hash of the remote file """
+        verified, hash_pair, calculated_hash = restore_check_hash(volume_info, tdp)
+        if verified:
+            break
+        else:
+            error_msg = u"%s\n %s\n %s\n %s\n" % (
+                _(u"Invalid data - %s hash mismatch for file:") %
+                hash_pair[0],
+                util.fsdecode(filename),
+                _(u"Calculated hash: %s") % calculated_hash,
+                _(u"Manifest hash: %s") % hash_pair[1]
+            )
+            log.Error(error_msg, code=log.ErrorCode.mismatched_hash)
+    else:
         if config.ignore_errors:
             exc = duplicity.errors.BadVolumeException(u"Hash mismatch for: %s" % util.fsdecode(filename))
-            log.Log(error_msg, log.ERROR, code=log.ErrorCode.mismatched_hash)
             log.Warn(_(u"IGNORED_ERROR: Warning: ignoring error as requested: %s: %s")
                      % (exc.__class__.__name__, util.uexc(exc)))
-            raise exc
         else:
             log.FatalError(error_msg, code=log.ErrorCode.mismatched_hash)
+
     fileobj = tdp.filtered_open_with_delete(u"rb")
     if parseresults.encrypted and config.gpg_profile.sign_key:
         restore_add_sig_check(fileobj)
@@ -1058,8 +1076,8 @@ def replicate():
 
     src_chainlist = src_stats.get_signature_chains(local=False, filelist=src_list)[0]
     tgt_chainlist = tgt_stats.get_signature_chains(local=False, filelist=tgt_list)[0]
-    sorted(src_chainlist, key=lambda chain: chain.start_time)
-    sorted(tgt_chainlist, key=lambda chain: chain.start_time)
+    src_chainlist = sorted(src_chainlist, key=lambda chain: chain.start_time)
+    tgt_chainlist = sorted(tgt_chainlist, key=lambda chain: chain.start_time)
     if not src_chainlist:
         log.Notice(_(u"No old backup sets found."))
         return
@@ -1096,8 +1114,8 @@ def replicate():
 
     src_chainlist = src_stats.get_backup_chains(filename_list=src_list)[0]
     tgt_chainlist = tgt_stats.get_backup_chains(filename_list=tgt_list)[0]
-    sorted(src_chainlist, key=lambda chain: chain.start_time)
-    sorted(tgt_chainlist, key=lambda chain: chain.start_time)
+    src_chainlist = sorted(src_chainlist, key=lambda chain: chain.start_time)
+    tgt_chainlist = sorted(tgt_chainlist, key=lambda chain: chain.start_time)
     for src_chain in src_chainlist:
         try:
             tgt_chain = list([chain for chain in tgt_chainlist if chain.start_time == src_chain.start_time])[0]
@@ -1289,7 +1307,7 @@ def sync_archive(col_stats):
             def __next__(self):
                 try:
                     res = Block(self.fileobj.read(self.get_read_size()))
-                except Exception:
+                except Exception as e:
                     if hasattr(self.fileobj, u'name'):
                         name = self.fileobj.name
                         # name may be a path
@@ -1297,8 +1315,7 @@ def sync_archive(col_stats):
                             name = name.name
                     else:
                         name = None
-                    log.FatalError(_(u"Failed to read %s: %s") %
-                                   (util.fsdecode(name), sys.exc_info()),
+                    log.FatalError(_(u"Failed to read %s: %s") % (util.fsdecode(fn), util.uexc(e)),
                                    log.ErrorCode.generic)
                 if not res.data:
                     self.fileobj.close()
@@ -1657,7 +1674,9 @@ def do_backup(action):
     elif action == u"list-current":
         list_current(col_stats)
     elif action == u"collection-status":
-        if not config.file_changed:
+        if config.show_changes_in_set is not None:
+            log.PrintCollectionChangesInSet(col_stats, config.show_changes_in_set, True)
+        elif not config.file_changed:
             log.PrintCollectionStatus(col_stats, True)
         else:
             log.PrintCollectionFileChangedStatus(col_stats, config.file_changed, True)
