@@ -1,4 +1,4 @@
-# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4 -*-
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
 #
 # Copyright 2002 Ben Escoto <ben@emerose.org>
 # Copyright 2007 Kenneth Loafman <kenneth@loafman.com>
@@ -19,7 +19,7 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
-"""
+u"""
 Duplicity specific but otherwise generic threading interfaces and
 utilities.
 
@@ -28,12 +28,18 @@ the standard threading module, and absolute imports require
 at least python 2.5.)
 """
 
+from future import standard_library
+standard_library.install_aliases()
+from builtins import object
+import sys
+from duplicity import errors
+
 _threading_supported = True
 
 try:
-    import thread
+    import _thread
 except ImportError:
-    import dummy_thread as thread
+    import _dummy_thread as _thread
     _threading_supported = False
 
 try:
@@ -42,19 +48,17 @@ except ImportError:
     import dummy_threading as threading
     _threading_supported = False
 
-import sys
-
-from duplicity import errors
 
 def threading_supported():
-    """
+    u"""
     Returns whether threading is supported on the system we are
     running on.
     """
     return _threading_supported
 
-def require_threading(reason = None):
-    """
+
+def require_threading(reason=None):
+    u"""
     Assert that threading is required for operation to continue. Raise
     an appropriate exception if this is not the case.
 
@@ -64,27 +68,30 @@ def require_threading(reason = None):
     """
     if not threading_supported():
         if reason is None:
-            reason = "(no reason given)"
-        raise errors.NotSupported("threading was needed because [%s], but "\
-                                      "is not supported by the python "\
-                                      "interpreter" % (reason,))
+            reason = u"(no reason given)"
+        raise errors.NotSupported(u"threading was needed because [%s], but "
+                                  u"is not supported by the python "
+                                  u"interpreter" % (reason,))
+
 
 def thread_module():
-    """
+    u"""
     Returns the thread module, or dummy_thread if threading is not
     supported.
     """
-    return thread
+    return _thread
+
 
 def threading_module():
-    """
+    u"""
     Returns the threading module, or dummy_thread if threading is not
     supported.
     """
     return threading
 
+
 def with_lock(lock, fn):
-    """
+    u"""
     Call fn with lock acquired. Guarantee that lock is released upon
     the return of fn.
 
@@ -101,8 +108,9 @@ def with_lock(lock, fn):
     finally:
         lock.release()
 
+
 def interruptably_wait(cv, waitFor):
-    """
+    u"""
     cv   - The threading.Condition instance to wait on
     test - Callable returning a boolean to indicate whether
            the criteria being waited on has been satisfied.
@@ -140,7 +148,6 @@ def interruptably_wait(cv, waitFor):
       cv.acquire()
       interruptable_condwait(cv, lambda: thing_done)
       cv.release()
-
     """
     # We can either poll at some interval, or wait with a short enough
     # timeout to be practical (i.e., such that it interactively seems
@@ -152,8 +159,9 @@ def interruptably_wait(cv, waitFor):
     while not waitFor():
         cv.wait(0.1)
 
+
 def async_split(fn):
-    """
+    u"""
     Splits the act of calling the given function into one front-end
     part for waiting on the result, and a back-end part for performing
     the work in another thread.
@@ -177,22 +185,21 @@ def async_split(fn):
     # not care about hash lookup overhead since this is intended to be
     # used for significant amounts of work.
 
-
-    cv = threading.Condition() #@UndefinedVariable
-    state = { 'done': False,
-              'error': None,
-              'trace': None,
-              'value': None }
+    cv = threading.Condition()
+    state = {u'done': False,
+             u'error': None,
+             u'trace': None,
+             u'value': None}
 
     def waiter():
         cv.acquire()
         try:
-            interruptably_wait(cv, lambda: state['done'])
+            interruptably_wait(cv, lambda: state[u'done'])
 
-            if state['error'] is None:
-                return state['value']
+            if state[u'error'] is None:
+                return state[u'value']
             else:
-                raise state['error'], None, state['trace']
+                raise state[u'error'].with_traceback(state[u'trace'])
         finally:
             cv.release()
 
@@ -201,17 +208,17 @@ def async_split(fn):
             value = fn()
 
             cv.acquire()
-            state['done'] = True
-            state['value'] = value
+            state[u'done'] = True
+            state[u'value'] = value
             cv.notify()
             cv.release()
 
             return (True, waiter)
-        except Exception, e:
+        except Exception as e:
             cv.acquire()
-            state['done'] = True
-            state['error'] = e
-            state['trace'] = sys.exc_info()[2]
+            state[u'done'] = True
+            state[u'error'] = e
+            state[u'trace'] = sys.exc_info()[2]
             cv.notify()
             cv.release()
 
@@ -219,8 +226,9 @@ def async_split(fn):
 
     return (waiter, caller)
 
-class Value:
-    """
+
+class Value(object):
+    u"""
     A thread-safe container of a reference to an object (but not the
     object itself).
 
@@ -244,22 +252,22 @@ class Value:
       value.transform(lambda val: val + 1)
     """
 
-    def __init__(self, value = None):
-        """
+    def __init__(self, value=None):
+        u"""
         Initialuze with the given value.
         """
         self.__value = value
 
-        self.__cv = threading.Condition() #@UndefinedVariable
+        self.__cv = threading.Condition()
 
     def get(self):
-        """
+        u"""
         Returns the value protected by this Value.
         """
         return with_lock(self.__cv, lambda: self.__value)
 
     def set(self, value):
-        """
+        u"""
         Resets the value protected by this Value.
         """
         def _set():
@@ -268,7 +276,7 @@ class Value:
         with_lock(self.__cv, _set)
 
     def transform(self, fn):
-        """
+        u"""
         Call fn with the current value as the parameter, and reset the
         value to the return value of fn.
 
@@ -287,7 +295,7 @@ class Value:
         return with_lock(self.cv, _transform)
 
     def acquire(self):
-        """
+        u"""
         Acquire this Value for mutually exclusive access. Only ever
         needed when calling code must perform operations that cannot
         be done with get(), set() or transform().
@@ -295,7 +303,7 @@ class Value:
         self.__cv.acquire()
 
     def release(self):
-        """
+        u"""
         Release this Value for mutually exclusive access.
         """
         self.__cv.release()
