@@ -1,9 +1,11 @@
-"""Interface to GNU Privacy Guard (GnuPG)
+# -*- Mode:Python; indent-tabs-mode:nil; tab-width:4; encoding:utf-8 -*-
+#
+u"""Interface to GNU Privacy Guard (GnuPG)
 
 !!! This was renamed to gpginterface.py.
     Please refer to duplicity's README for the reason. !!!
 
-gpginterface is a Python module to interface with GnuPG which based on 
+gpginterface is a Python module to interface with GnuPG which based on
 GnuPGInterface by Frank J. Tobin.
 It concentrates on interacting with GnuPG via filehandles,
 providing access to control GnuPG via versatile and extensible means.
@@ -224,6 +226,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 or see http://www.gnu.org/copyleft/lesser.html
 """
 
+from builtins import object
 import os
 import sys
 import fcntl
@@ -233,34 +236,36 @@ from duplicity import log
 try:
     import threading
 except ImportError:
-    import dummy_threading #@UnusedImport
-    log.Warn("Threading not available -- zombie processes may appear")
+    import dummy_threading as threading
+    log.Warn(_(u"Threading not available -- zombie processes may appear"))
 
-__author__   = "Frank J. Tobin, ftobin@neverending.org"
-__version__  = "0.3.2"
-__revision__ = "$Id: GnuPGInterface.py,v 1.6 2009/06/06 17:35:19 loafman Exp $"
+__author__ = u"Frank J. Tobin, ftobin@neverending.org"
+__version__ = u"0.3.2"
+__revision__ = u"$Id: GnuPGInterface.py,v 1.6 2009/06/06 17:35:19 loafman Exp $"
 
 # "standard" filehandles attached to processes
-_stds = [ 'stdin', 'stdout', 'stderr' ]
+_stds = [u'stdin', u'stdout', u'stderr']
 
 # the permissions each type of fh needs to be opened with
-_fd_modes = { 'stdin':      'w',
-              'stdout':     'r',
-              'stderr':     'r',
-              'passphrase': 'w',
-              'command':    'w',
-              'logger':     'r',
-              'status':     'r'
-              }
+_fd_modes = {u'stdin': u'wb',
+             u'stdout': u'rb',
+             u'stderr': u'r',
+             u'passphrase': u'w',
+             u'command': u'w',
+             u'logger': u'r',
+             u'status': u'r'
+             }
 
 # correlation between handle names and the arguments we'll pass
-_fd_options = { 'passphrase': '--passphrase-fd',
-                'logger':     '--logger-fd',
-                'status':     '--status-fd',
-                'command':    '--command-fd' }
+_fd_options = {u'passphrase': u'--passphrase-fd',
+               u'logger': u'--logger-fd',
+               u'status': u'--status-fd',
+               u'command': u'--command-fd'
+               }
 
-class GnuPG:
-    """Class instances represent GnuPG.
+
+class GnuPG(object):
+    u"""Class instances represent GnuPG.
 
     Instance attributes of a GnuPG object are:
 
@@ -281,12 +286,12 @@ class GnuPG:
     """
 
     def __init__(self):
-        self.call = 'gpg'
+        self.call = u'gpg'
         self.passphrase = None
         self.options = Options()
 
     def run(self, gnupg_commands, args=None, create_fhs=None, attach_fhs=None):
-        """Calls GnuPG with the list of string commands gnupg_commands,
+        u"""Calls GnuPG with the list of string commands gnupg_commands,
         complete with prefixing dashes.
         For example, gnupg_commands could be
         '["--sign", "--encrypt"]'
@@ -348,81 +353,85 @@ class GnuPG:
         and can be written to.
         """
 
-        if args == None: args = []
-        if create_fhs == None: create_fhs = []
-        if attach_fhs == None: attach_fhs = {}
+        if args is None:
+            args = []
+        if create_fhs is None:
+            create_fhs = []
+        if attach_fhs is None:
+            attach_fhs = {}
 
         for std in _stds:
-            if not attach_fhs.has_key(std) \
+            if std not in attach_fhs \
                and std not in create_fhs:
                 attach_fhs.setdefault(std, getattr(sys, std))
 
         handle_passphrase = 0
 
-        if self.passphrase != None \
-           and not attach_fhs.has_key('passphrase') \
-           and 'passphrase' not in create_fhs:
+        if self.passphrase is not None \
+           and u'passphrase' not in attach_fhs \
+           and u'passphrase' not in create_fhs:
             handle_passphrase = 1
-            create_fhs.append('passphrase')
+            create_fhs.append(u'passphrase')
 
         process = self._attach_fork_exec(gnupg_commands, args,
                                          create_fhs, attach_fhs)
 
         if handle_passphrase:
-            passphrase_fh = process.handles['passphrase']
-            passphrase_fh.write( self.passphrase )
+            passphrase_fh = process.handles[u'passphrase']
+            passphrase_fh.write(self.passphrase)
             passphrase_fh.close()
-            del process.handles['passphrase']
+            del process.handles[u'passphrase']
 
         return process
 
-
     def _attach_fork_exec(self, gnupg_commands, args, create_fhs, attach_fhs):
-        """This is like run(), but without the passphrase-helping
+        u"""This is like run(), but without the passphrase-helping
         (note that run() calls this)."""
 
         process = Process()
 
-        for fh_name in create_fhs + attach_fhs.keys():
-            if not _fd_modes.has_key(fh_name):
-                raise KeyError, \
-                      "unrecognized filehandle name '%s'; must be one of %s" \
-                      % (fh_name, _fd_modes.keys())
+        for fh_name in create_fhs + list(attach_fhs.keys()):
+            if fh_name not in _fd_modes:
+                raise KeyError(u"unrecognized filehandle name '%s'; must be one of %s"
+                               % (fh_name, list(_fd_modes.keys())))
 
         for fh_name in create_fhs:
             # make sure the user doesn't specify a filehandle
             # to be created *and* attached
-            if attach_fhs.has_key(fh_name):
-                raise ValueError, \
-                      "cannot have filehandle '%s' in both create_fhs and attach_fhs" \
-                      % fh_name
+            if fh_name in attach_fhs:
+                raise ValueError(u"cannot have filehandle '%s' in both create_fhs and attach_fhs"
+                                 % fh_name)
 
             pipe = os.pipe()
             # fix by drt@un.bewaff.net noting
             # that since pipes are unidirectional on some systems,
             # so we have to 'turn the pipe around'
             # if we are writing
-            if _fd_modes[fh_name] == 'w': pipe = (pipe[1], pipe[0])
+            if _fd_modes[fh_name] == u'w' or _fd_modes[fh_name] == u'wb':
+                pipe = (pipe[1], pipe[0])
+            if sys.version_info.major >= 3:
+                os.set_inheritable(pipe[0], True)
+                os.set_inheritable(pipe[1], True)
             process._pipes[fh_name] = Pipe(pipe[0], pipe[1], 0)
 
-        for fh_name, fh in attach_fhs.items():
+        for fh_name, fh in list(attach_fhs.items()):
             process._pipes[fh_name] = Pipe(fh.fileno(), fh.fileno(), 1)
 
         process.pid = os.fork()
         if process.pid != 0:
             # start a threaded_waitpid on the child
             process.thread = threading.Thread(target=threaded_waitpid,
-                                              name="wait%d" % process.pid,
+                                              name=u"wait%d" % process.pid,
                                               args=(process,))
             process.thread.start()
 
-        if process.pid == 0: self._as_child(process, gnupg_commands, args)
+        if process.pid == 0:
+            self._as_child(process, gnupg_commands, args)
         return self._as_parent(process)
 
-
     def _as_parent(self, process):
-        """Stuff run after forking in parent"""
-        for k, p in process._pipes.items():
+        u"""Stuff run after forking in parent"""
+        for k, p in list(process._pipes.items()):
             if not p.direct:
                 os.close(p.child)
                 process.handles[k] = os.fdopen(p.parent, _fd_modes[k])
@@ -432,44 +441,43 @@ class GnuPG:
 
         return process
 
-
     def _as_child(self, process, gnupg_commands, args):
-        """Stuff run after forking in child"""
+        u"""Stuff run after forking in child"""
         # child
         for std in _stds:
             p = process._pipes[std]
-            os.dup2( p.child, getattr(sys, "__%s__" % std).fileno() )
+            os.dup2(p.child, getattr(sys, u"__%s__" % std).fileno())
 
-        for k, p in process._pipes.items():
+        for k, p in list(process._pipes.items()):
             if p.direct and k not in _stds:
                 # we want the fh to stay open after execing
-                fcntl.fcntl( p.child, fcntl.F_SETFD, 0 )
+                fcntl.fcntl(p.child, fcntl.F_SETFD, 0)
 
         fd_args = []
 
-        for k, p in process._pipes.items():
+        for k, p in list(process._pipes.items()):
             # set command-line options for non-standard fds
             if k not in _stds:
-                fd_args.extend([ _fd_options[k], "%d" % p.child ])
+                fd_args.extend([_fd_options[k], u"%d" % p.child])
 
-            if not p.direct: os.close(p.parent)
+            if not p.direct:
+                os.close(p.parent)
 
-        command = [ self.call ] + fd_args + self.options.get_args() \
-                  + gnupg_commands + args
+        command = [self.call] + fd_args + self.options.get_args() + gnupg_commands + args
 
-        os.execvp( command[0], command )
+        os.execvp(command[0], command)
 
 
-class Pipe:
-    """simple struct holding stuff about pipes we use"""
+class Pipe(object):
+    u"""simple struct holding stuff about pipes we use"""
     def __init__(self, parent, child, direct):
         self.parent = parent
         self.child = child
         self.direct = direct
 
 
-class Options:
-    """Objects of this class encompass options passed to GnuPG.
+class Options(object):
+    u"""Objects of this class encompass options passed to GnuPG.
     This class is responsible for determining command-line arguments
     which are based on options.  It can be said that a GnuPG
     object has-a Options object in its options attribute.
@@ -575,54 +583,76 @@ class Options:
         # miscellaneous arguments
         self.extra_args = []
 
-    def get_args( self ):
-        """Generate a list of GnuPG arguments based upon attributes."""
+    def get_args(self):
+        u"""Generate a list of GnuPG arguments based upon attributes."""
 
         return self.get_meta_args() + self.get_standard_args() + self.extra_args
 
-    def get_standard_args( self ):
-        """Generate a list of standard, non-meta or extra arguments"""
+    def get_standard_args(self):
+        u"""Generate a list of standard, non-meta or extra arguments"""
         args = []
-        if self.homedir != None: args.extend( [ '--homedir', self.homedir ] )
-        if self.options != None: args.extend( [ '--options', self.options ] )
-        if self.comment != None: args.extend( [ '--comment', self.comment ] )
-        if self.compress_algo != None: args.extend( [ '--compress-algo', self.compress_algo ] )
-        if self.default_key != None: args.extend( [ '--default-key', self.default_key ] )
+        if self.homedir is not None:
+            args.extend([u'--homedir', self.homedir])
+        if self.options is not None:
+            args.extend([u'--options', self.options])
+        if self.comment is not None:
+            args.extend([u'--comment', self.comment])
+        if self.compress_algo is not None:
+            args.extend([u'--compress-algo', self.compress_algo])
+        if self.default_key is not None:
+            args.extend([u'--default-key', self.default_key])
 
-        if self.no_options: args.append( '--no-options' )
-        if self.armor: args.append( '--armor' )
-        if self.textmode: args.append( '--textmode' )
-        if self.no_greeting: args.append( '--no-greeting' )
-        if self.verbose: args.append( '--verbose' )
-        if self.no_verbose: args.append( '--no-verbose' )
-        if self.quiet: args.append( '--quiet' )
-        if self.batch: args.append( '--batch' )
-        if self.always_trust: args.append( '--always-trust' )
-        if self.force_v3_sigs: args.append( '--force-v3-sigs' )
-        if self.rfc1991: args.append( '--rfc1991' )
-        if self.openpgp: args.append( '--openpgp' )
+        if self.no_options:
+            args.append(u'--no-options')
+        if self.armor:
+            args.append(u'--armor')
+        if self.textmode:
+            args.append(u'--textmode')
+        if self.no_greeting:
+            args.append(u'--no-greeting')
+        if self.verbose:
+            args.append(u'--verbose')
+        if self.no_verbose:
+            args.append(u'--no-verbose')
+        if self.quiet:
+            args.append(u'--quiet')
+        if self.batch:
+            args.append(u'--batch')
+        if self.always_trust:
+            args.append(u'--always-trust')
+        if self.force_v3_sigs:
+            args.append(u'--force-v3-sigs')
+        if self.rfc1991:
+            args.append(u'--rfc1991')
+        if self.openpgp:
+            args.append(u'--openpgp')
 
-        for r in self.recipients: args.extend( [ '--recipient',  r ] )
-        for r in self.hidden_recipients: args.extend( [ '--hidden-recipient',  r ] )
-        for r in self.encrypt_to: args.extend( [ '--encrypt-to', r ] )
+        for r in self.recipients:
+            args.extend([u'--recipient', r])
+        for r in self.hidden_recipients:
+            args.extend([u'--hidden-recipient', r])
+        for r in self.encrypt_to:
+            args.extend([u'--encrypt-to', r])
 
         return args
 
-    def get_meta_args( self ):
-        """Get a list of generated meta-arguments"""
+    def get_meta_args(self):
+        u"""Get a list of generated meta-arguments"""
         args = []
 
-        if self.meta_pgp_5_compatible: args.extend( [ '--compress-algo', '1',
-                                                      '--force-v3-sigs'
-                                                      ] )
-        if self.meta_pgp_2_compatible: args.append( '--rfc1991' )
-        if not self.meta_interactive: args.extend( [ '--batch', '--no-tty' ] )
+        if self.meta_pgp_5_compatible:
+            args.extend([u'--compress-algo', u'1',
+                         u'--force-v3-sigs'])
+        if self.meta_pgp_2_compatible:
+            args.append(u'--rfc1991')
+        if not self.meta_interactive:
+            args.extend([u'--batch', u'--no-tty'])
 
         return args
 
 
-class Process:
-    """Objects of this class encompass properties of a GnuPG
+class Process(object):
+    u"""Objects of this class encompass properties of a GnuPG
     process spawned by GnuPG.run().
 
     # gnupg is a GnuPG object
@@ -645,26 +675,26 @@ class Process:
     """
 
     def __init__(self):
-        self._pipes   = {}
-        self.handles  = {}
-        self.pid      = None
-        self._waited  = None
-        self.thread   = None
+        self._pipes = {}
+        self.handles = {}
+        self.pid = None
+        self._waited = None
+        self.thread = None
         self.returned = None
 
     def wait(self):
-        """
+        u"""
         Wait on threaded_waitpid to exit and examine results.
         Will raise an IOError if the process exits non-zero.
         """
-        if self.returned == None:
+        if self.returned is None:
             self.thread.join()
         if self.returned != 0:
-            raise IOError, "GnuPG exited non-zero, with code %d" % (self.returned >> 8)
+            raise IOError(u"GnuPG exited non-zero, with code %d" % (self.returned >> 8))
 
 
 def threaded_waitpid(process):
-    """
+    u"""
     When started as a thread with the Process object, thread
     will execute an immediate waitpid() against the process
     pid and will collect the process termination info.  This
@@ -674,16 +704,17 @@ def threaded_waitpid(process):
     try:
         process.returned = os.waitpid(process.pid, 0)[1]
     except:
-        log.Debug("GPG process %d terminated before wait()" % process.pid)
+        log.Debug(_(u"GPG process %d terminated before wait()") % process.pid)
         process.returned = 0
 
 
 def _run_doctests():
-    import doctest, gpginterface #@UnresolvedImport
+    import doctest
     return doctest.testmod(GnuPGInterface)
 
+
 # deprecated
 GnuPGInterface = GnuPG
 
-if __name__ == '__main__':
+if __name__ == u'__main__':
     _run_doctests()
